<?php

namespace Drupal\entity_reference_facet_link\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\facets\Result\Result;
use Drupal\facets\UrlProcessor\UrlProcessorPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity reference facet link' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_facet_link",
 *   label = @Translation("Facet link"),
 *   description = @Translation("Display the label of the referenced entities linked to a faceted search."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceFacetLink extends EntityReferenceFormatterBase implements ContainerFactoryPluginInterface {

  /**
   * A URL processor plugin manager.
   *
   * @var \Drupal\facets\UrlProcessor\UrlProcessorPluginManager
   */
  protected $urlProcessorPluginManager;

  /**
   * Constructs a EntityReferenceFacetLink object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\facets\UrlProcessor\UrlProcessorPluginManager $url_processor_plugin_manager
   *   A URL processor plugin manager.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, UrlProcessorPluginManager $url_processor_plugin_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->urlProcessorPluginManager = $url_processor_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('plugin.manager.facets.url_processor')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
        'facet' => '',
      ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\facets\FacetInterface[] $facets */
    $facets = \Drupal::entityTypeManager()->getStorage('facets_facet')->loadMultiple();
    $options = [];
    foreach ($facets as $facet) {
      // Add a facet to the options only if that facet is faceting this field.
      if ($facet->getFieldIdentifier() == $this->fieldDefinition->getName()) {
        $options[$facet->id()] = $facet->label();
      }
    }

    $elements['facet'] = array(
      '#title' => t('Select the facet to which the labels should be linked.'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('facet'),
      '#options' => $options,
    );

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if ($this->getSetting('facet')) {
      /** @var \Drupal\facets\FacetInterface $facet */
      $facet = \Drupal::entityTypeManager()->getStorage('facets_facet')->load($this->getSetting('facet'));
      $summary[] = t('Link to facet: @facet', ['@facet' => $facet->label()]);
    }
    else {
      $summary[] = t('No facet selected');
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    /** @var \Drupal\facets\FacetInterface $facet */
    $facet = \Drupal::entityTypeManager()->getStorage('facets_facet')->load($this->getSetting('facet'));

    if ($facet) {
      // Instead of trying to guess how the facet URLs should be formatted, let
      // the facet's own URL processor do the work of building them.  Then the
      // URLs will be formatted correctly no matter what processor is being
      // used, for instance Facet Pretty Paths.
      $url_processor_id = $facet->getFacetSourceConfig()->getUrlProcessorName();
      $configuration = ['facet' => $facet];
      /** @var \Drupal\facets\UrlProcessor\UrlProcessorInterface $url_processor */
      $url_processor = $this->urlProcessorPluginManager->createInstance($url_processor_id, $configuration);

      foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
        // Create a fake Result object from the field item so that we can pass
        // it to the URL processor.
        $result = new Result($entity->id(), $entity->label(), 0);
        $result = $url_processor->buildUrls($facet, [$result])[0];

        // Invalidate the cache when the referenced entity or the facet source
        // config changes.  The source display config, for instance a view,
        // should be added here too, but there really isn't any way to access
        // that config entity through the API.
        $cache_tags = Cache::mergeTags($entity->getCacheTags(), $facet->getFacetSourceConfig()->getCacheTags());

        $elements[$delta] = [
          '#type' => 'link',
          '#title' => $entity->label(),
          '#url' => $result->getUrl(),
          '#cache' => [
            'tags' => $cache_tags,
          ],
        ];
      }
    }

    return $elements;
  }

}
